#include "tests.h"

int main() {
    int failed_count = 0;
    int size[TEST_COUNT] = {256, 512, 1060, 2126, 8192};

    int print = 0;

    print_table_header();

    #ifndef LOCAL_MEM
        void* src0 = memalign(64, 2 * size[TEST_COUNT - 1] * sizeof(int32_t));
        void* src1 = memalign(64, 2 * size[TEST_COUNT - 1] * sizeof(int32_t));
        void* dst_opt = memalign(64, 2 * size[TEST_COUNT - 1] * sizeof(int32_t));
        void* dst_ref = memalign(64, 2 * size[TEST_COUNT - 1] * sizeof(int32_t));
    #else
        #ifdef BARE_METAL
        void* src0 = &__local_mem;
        #else
        uint32_t l2dc_val = *L2DC;
        FLUSH_ALL_CACHES();
        *L2DC &= 0xFFFFFFBF;
        void* src0 = &xyram_data;
        #endif
        void* src1 = src0 + 2 * size[TEST_COUNT - 1] * sizeof(int32_t);
        void* dst_opt = src1 + 2 * size[TEST_COUNT - 1] * sizeof(int32_t);
        void* dst_ref = dst_opt + 2 * size[TEST_COUNT - 1] * sizeof(int32_t);
    #endif

    create_vector_s32(src0, 2 * size[TEST_COUNT - 1], 0);
    create_vector_s32(src1, 2 * size[TEST_COUNT - 1], 0);

    memset(dst_ref, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);
    memset(dst_opt, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);

    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 2;
        printf("| add_cplx32             | %14d |", size[i]);

        failed_count +=
            test_add_cplx32((int32_t*)src0, (int32_t*)src1, (int32_t*)dst_opt,
                              (int32_t*)dst_ref, size[i], print, input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");
    memset(dst_ref, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);
    memset(dst_opt, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);
    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 2;
        printf("| vec_acc_cplx32         | %14d |", size[i]);

        failed_count +=
            test_vec_acc_cplx32((int32_t*)src0, (int32_t*)dst_opt,
                              (int32_t*)dst_ref, size[i], print, input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");
    memset(dst_ref, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);
    memset(dst_opt, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);
    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 2;
        printf("| sub_cplx32             | %14d |", size[i]);

        failed_count +=
            test_sub_cplx32((int32_t*)src0, (int32_t*)src1, (int32_t*)dst_opt,
                              (int32_t*)dst_ref, size[i], print, input_bytes);
    }

    for (int i = 0; i < size[TEST_COUNT - 1] * 2; ++i) {
        ((int32_t*)src0)[i] = rand()%100 - 50;
        ((int32_t*)src1)[i] = rand()%100 - 50;
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");
    memset(dst_ref, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);
    memset(dst_opt, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);
    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 2;
        printf("| mul_cplx32             | %14d |", size[i]);

        failed_count +=
            test_mul_cplx32((int32_t*)src0, (int32_t*)src1, (int32_t*)dst_opt,
                              (int32_t*)dst_ref, size[i], print, input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");
    memset(dst_ref, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);
    memset(dst_opt, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);
    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 2;
        printf("| mul_conj_cplx32        | %14d |", size[i]);

        failed_count +=
            test_mul_conj_cplx32(
                (int32_t*)src0, (int32_t*)src1, (int32_t*)dst_opt,
                (int32_t*)dst_ref, size[i], print, input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");
    create_vector_s32(dst_ref, 2 * size[TEST_COUNT - 1], 0);
    memcpy(dst_opt, dst_ref, size[TEST_COUNT - 1] * 2 * sizeof(int32_t));

    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 1;
        printf("| add_rconst_cplx32      | %14d |", size[i]);

        failed_count +=
            test_add_rconst_cplx32(((int32_t*)src0)[0], (int32_t*)dst_opt,
                                     (int32_t*)dst_ref, size[i], print,
                                     input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");

    for (int i = 0; i < size[TEST_COUNT - 1] * 2; ++i) {
        ((int32_t*)dst_ref)[i] = rand()%100 - 50;
        ((int32_t*)dst_opt)[i] = ((int32_t*)dst_ref)[i];
    }

    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 1;
        printf("| mul_rconst_cplx32      | %14d |", size[i]);
        int32_t coef = rand()%10 - 5;
        failed_count +=
            test_mul_rconst_cplx32(coef, (int32_t*)dst_opt, (int32_t*)dst_ref,
                                     size[i], print, input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");
    create_vector_s32(dst_ref, 2 * size[TEST_COUNT - 1], 0);
    memcpy(dst_opt, dst_ref, size[TEST_COUNT - 1] * 2 * sizeof(int32_t));

    int32_t scale = 1;
    for(int i = 0; i < TEST_COUNT; ++i) {
        if (i % 2)
            scale = -scale;
        else
            scale = rand()%10 - 5;

        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 1;
        printf("| scale_cplx32           | %14d |", size[i]);

        failed_count +=
            test_scale_cplx32(scale, (int32_t*)dst_opt,
                                (int32_t*)dst_ref, size[i], print,
                                input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");

    for (int i = 0; i < size[TEST_COUNT - 1] * 2; ++i) {
        ((int32_t*)dst_ref)[i] = rand()%100 - 50;
        ((int32_t*)dst_opt)[i] = ((int32_t*)dst_ref)[i];
    }

    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 1;
        printf("| div_rconst_cplx32      | %14d |", size[i]);
        int32_t coef = rand()%10 - 5;
        if (coef == 0)
            coef = rand()%10 + 1;

        failed_count +=
            test_div_rconst_cplx32(coef, (int32_t*)dst_opt, (int32_t*)dst_ref,
                                     size[i], print, input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");

    for (int i = 0; i < size[TEST_COUNT - 1] * 2; ++i) {
        ((int32_t*)dst_ref)[i] = rand()%10 - 5;
        ((int32_t*)dst_opt)[i] = ((int32_t*)dst_ref)[i];
    }

    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 1;
        printf("| vec_sum_cplx32         | %14d |", size[i]);

        failed_count +=
            test_vec_sum_cplx32((int32_t*)dst_opt, (int32_t*)dst_ref, size[i],
                              print, input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");

    for (int i = 0; i < size[TEST_COUNT - 1] * 2; ++i) {
        ((int32_t*)src0)[i] = rand()%100 - 50;
    }

    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 1;
        printf("| abs_cplx32             | %14d |", size[i]);
        failed_count +=
            test_abs_cplx32(src0, (int32_t*)dst_opt, (int32_t*)dst_ref,
                              size[i], print, input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");

    memset(dst_ref, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);
    memset(dst_opt, 0, sizeof(int32_t) * size[TEST_COUNT - 1] * 2);

    for (int i = 0; i < size[TEST_COUNT - 1] * 2; ++i) {
        ((int32_t*)src0)[i] = rand()%4 - 2;
    }

    for (int i = 0; i < size[TEST_COUNT - 1] * 2; ++i) {
        ((int32_t*)src1)[i] = rand()%4 - 2;
    }

    for(int i = 0; i < TEST_COUNT; ++i) {
        int32_t input_bytes = size[i] * 2 * sizeof(int32_t) * 2;
        int size_ker = (rand() % size[i]) + 1;
        printf("| conv_cplx, ker = %5d | %14d |", size_ker, size[i]);

        failed_count +=
            test_conv_cplx32((int32_t*)src0, (int32_t*)src1, (int32_t*)dst_opt,
                           (int32_t*)dst_ref, size[i], size_ker, print, input_bytes);
    }

    printf("-------------------------------------------------------------------"
           "------------------------------------------------------\n");

    #ifndef LOCAL_MEM
        free(src0);
        free(src1);
        free(dst_opt);
        free(dst_ref);
    #else
    #ifndef BARE_METAL
        *L2DC = l2dc_val;
    #endif
    #endif

    return failed_count;
}

int test_add_cplx32(
    int32_t* src0, int32_t* src1, int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {
    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    ref_add_cplx32(src0, src1, size, dst_ref);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    add_cplx32(src0, src1, size, dst_opt);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s32(src0, size);
        printf("vect2:");
        print_vector_s32(src1, size);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size);
        printf("ref_res:");
        print_vector_s32(dst_ref, size);
    }

    ret = compare_s32(dst_ref, dst_opt, size * 2);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_vec_acc_cplx32(int32_t* src0, int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {
    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    ref_acc_cplx32(dst_ref, src0, size);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    acc_cplx32(dst_opt, src0, size);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s32(src0, size);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size);
        printf("ref_res:");
        print_vector_s32(dst_ref, size);
    }

    ret = compare_s32(dst_ref, dst_opt, size * 2);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_sub_cplx32(
        int32_t* src0, int32_t* src1, int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {

    int ret = 0;
    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    ref_sub_cplx32(src0, src1, size, dst_ref);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    sub_cplx32(src0, src1, size, dst_opt);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s32(src0, size);
        printf("vect2:");
        print_vector_s32(src1, size);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size);
        printf("ref_res:");
        print_vector_s32(dst_ref, size);
    }

    ret = compare_s32(dst_ref, dst_opt, size * 2);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_mul_cplx32(
        int32_t* src0, int32_t* src1, int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {

    int ret = 0;
    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    ref_mul_cplx32(src0, src1, size, dst_ref);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    mul_cplx32(src0, src1, size, dst_opt);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s32(src0, size);
        printf("vect2:");
        print_vector_s32(src1, size);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size);
        printf("ref_res:");
        print_vector_s32(dst_ref, size);
    }

    ret = compare_s32(dst_ref, dst_opt, size * 2);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_mul_conj_cplx32(
    int32_t* src0, int32_t* src1, int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {

    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    ref_mul_conj_cplx32(src0, src1, size, dst_ref);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    mul_conj_cplx32(src0, src1, size, dst_opt);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s32(src0, size);
        printf("vect2:");
        print_vector_s32(src1, size);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size);
        printf("ref_res:");
        print_vector_s32(dst_ref, size);
    }

    ret = compare_s32(dst_ref, dst_opt, size * 2);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_add_rconst_cplx32(int32_t src0, int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {

    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    ref_add_rconst_cplx32(dst_ref, size, src0);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    add_rconst_cplx32(dst_opt, size, src0);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s32(&src0, 1);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size);
        printf("ref_res:");
        print_vector_s32(dst_ref, size);
    }

    ret = compare_s32(dst_ref, dst_opt, size * 2);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_mul_rconst_cplx32(int32_t src0, int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {

    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    ref_mul_rconst_cplx32(dst_ref, size, src0);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    mul_rconst_cplx32(dst_opt, size, src0);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s32(&src0, 1);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size);
        printf("ref_res:");
        print_vector_s32(dst_ref, size);
    }

    ret = compare_s32(dst_ref, dst_opt, size * 2);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_scale_cplx32(int32_t src0, int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {

    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    if (print)
    {
        printf("\nvect1:");
        print_vector_s32(&src0, 1);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size * 2);
        printf("ref_res:");
        print_vector_s32(dst_ref, size * 2);
        printf("after:\n");
    }

    count_tics(ref_tic_count, ref_instruction_count);
    ref_scale_cplx32(dst_ref, size, src0);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    scale_cplx32(dst_opt, size, src0);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("\nvect1:");
        print_vector_s32(&src0, 1);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size * 2);
        printf("ref_res:");
        print_vector_s32(dst_ref, size * 2);
    }

    ret = compare_s32(dst_ref, dst_opt, size * 2);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_div_rconst_cplx32(int32_t src0, int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {
    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    if (print)
    {
        printf("vect1:");
        print_vector_s32(&src0, 1);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size);
        printf("ref_res:");
        print_vector_s32(dst_ref, size);
    }

    count_tics(ref_tic_count, ref_instruction_count);
    ref_div_rconst_cplx32(dst_ref, size, src0);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    div_rconst_cplx32(dst_opt, size, src0);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s32(&src0, 1);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size);
        printf("ref_res:");
        print_vector_s32(dst_ref, size);
    }

    ret = compare_s32(dst_ref, dst_opt, size * 2);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_vec_sum_cplx32(int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {

    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    int32_t ref_res[2];
    count_tics(ref_tic_count, ref_instruction_count);
    ref_sum_cplx32(dst_ref, size, ref_res);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    int32_t opt_res[2];
    count_tics(tic_count, instruction_count);
    sum_cplx32(dst_opt, size, opt_res);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("dsp_res:");
        print_vector_s32(opt_res, size);
        printf("ref_res:");
        print_vector_s32(ref_res, size);
    }

    ret = compare_s32(ref_res, opt_res, 2);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_abs_cplx32(int32_t* src0, int32_t* dst_opt, int32_t* dst_ref, int size, int print, int32_t input_bytes) {

    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    ref_abs_cplx32(src0, size, dst_ref);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    abs_cplx32(src0, size, dst_opt);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s32(src0, size * 2);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size);
        printf("ref_res:");
        print_vector_s32(dst_ref, size);
    }

    ret = compare_s32_eps(dst_ref, dst_opt, size, 0.01);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, 0);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_conv_cplx32(int32_t* src0, int32_t* src1, int32_t* dst_opt, int32_t* dst_ref, int size, int size_ker,
                     int print, int32_t input_bytes) {
    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    ref_conv_cplx32(src0, src1, size, size_ker, dst_ref);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    conv_cplx32(src0, src1, size, size_ker, dst_opt);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (0)
    {
        printf("vect1:");
        print_vector_s32(src0, size * 2);
        printf("vect2:");
        print_vector_s32(src1, size * 2);
        printf("dsp_res:");
        print_vector_s32(dst_opt, size * 2);
        printf("ref_res:");
        print_vector_s32(dst_ref, size * 2);
    }

    ret = compare_s32(dst_ref, dst_opt, size * 2);

    printf(" %10d |", ref_tic_count[1] - ref_tic_count[0]);
    #ifdef PRINT_INSTR
    printf(" %10d |", ref_instruction_count[1] - ref_instruction_count[0]);
    #endif
    printf(" %10d |", tic_count[1] - tic_count[0]);
    #ifdef PRINT_INSTR
    printf(" %10d |", instruction_count[1] - instruction_count[0]);
    #endif

    printf("          - |");
    printf(" %4.1f mul/tic |", 1.0 * size_ker * size / (tic_count[1] - tic_count[0]));
    printf("            - |");

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;

}
